using System ;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Reflection;


namespace MagniMan
{
	/// <summary>
	/// Zusammenfassung fr MouseHook.
	/// </summary>
	public class MouseHook : object
	{
		public MouseHook()
		{
			Start() ;
		}

		~MouseHook()
		{
			Stop() ;
		}

		public event MouseEventHandler OnMouseActivity ;
		

		public delegate int HookProc( int nCode , Int32 wParam , IntPtr lParam ) ;

		static int hMouseHook = 0 ;

		//Werte aus der Winuser.h der Microsoft SDK.
		public const int WH_MOUSE_LL = 14 ;

		HookProc MouseHookProcedure ; //Declare MouseHookProcedure as HookProc type.

		//Declare wrapper managed POINT class.
		[StructLayout( LayoutKind.Sequential )]
			public class POINT
		{
			public int x ;
			public int y ;
		}

		//Declare wrapper managed MouseHookStruct class.
		[StructLayout( LayoutKind.Sequential )]
		public class MouseHookStruct
		{
			public		POINT pt ;
			public int	hwnd ;
			public int	wHitTestCode ;
			public int	dwExtraInfo ;
		}


		//Import for SetWindowsHookEx function.
		//Use this function to install a hook.
		[DllImport( "user32.dll" , CharSet = CharSet.Auto , CallingConvention = CallingConvention.StdCall )]
		public static extern int SetWindowsHookEx( int idHook , HookProc lpfn , IntPtr hInstance , int threadId ) ;

		//Import for UnhookWindowsHookEx.
		//Call this function to uninstall the hook.
		[DllImport( "user32.dll" , CharSet = CharSet.Auto , CallingConvention = CallingConvention.StdCall )]
		public static extern bool UnhookWindowsHookEx( int idHook ) ;

		//Import for CallNextHookEx.
		//Use this function to pass the hook information to next hook procedure in chain.
		[DllImport( "user32.dll" , CharSet = CharSet.Auto , CallingConvention = CallingConvention.StdCall )]
		public static extern int CallNextHookEx( int idHook , int nCode , Int32 wParam , IntPtr lParam ) ;

		public void Start()
		{
			// install Mouse hook 
			if( hMouseHook == 0 )
			{
				// Create an instance of HookProc.
				MouseHookProcedure = new HookProc( MouseHookProc ) ;

				hMouseHook = SetWindowsHookEx( WH_MOUSE_LL , MouseHookProcedure , Marshal.GetHINSTANCE( Assembly.GetExecutingAssembly().GetModules()[0] ) , 0 ) ;

				//If SetWindowsHookEx fails.
				if( hMouseHook == 0 )
				{
					Stop() ;
					//throw new Exception( "SetWindowsHookEx failed." );
				}
			}
		}

		public void Stop()
		{
			bool retMouse = true ;

			if( hMouseHook != 0 )
			{
				retMouse = UnhookWindowsHookEx( hMouseHook ) ;
				hMouseHook = 0 ;
			}

			//If UnhookWindowsHookEx fails.
			if( !retMouse )
			{
				throw new Exception( "UnhookWindowsHookEx failed." ) ;
			}
		}

		private const int WM_MOUSEMOVE		= 0x200 ;
		private const int WM_LBUTTONDOWN	= 0x201 ;
		private const int WM_RBUTTONDOWN	= 0x204 ;
		private const int WM_MBUTTONDOWN	= 0x207 ;
		private const int WM_LBUTTONUP		= 0x202 ;
		private const int WM_RBUTTONUP		= 0x205 ;
		private const int WM_MBUTTONUP		= 0x208 ;
		private const int WM_LBUTTONDBLCLK	= 0x203 ;
		private const int WM_RBUTTONDBLCLK	= 0x206 ;
		private const int WM_MBUTTONDBLCLK	= 0x209 ;

		private int MouseHookProc( int nCode , Int32 wParam , IntPtr lParam )
		{
			// if ok and someone listens to our events
			if( ( nCode >= 0 ) && ( OnMouseActivity != null ) )
			{
				MouseButtons button = MouseButtons.None ;

				switch( wParam )
				{
					case WM_LBUTTONDOWN:
						//case WM_LBUTTONUP: 
						//case WM_LBUTTONDBLCLK: 
						button = MouseButtons.Left ;
						break ;
					case WM_RBUTTONDOWN:
						//case WM_RBUTTONUP: 
						//case WM_RBUTTONDBLCLK: 
						button = MouseButtons.Right ;
						break ;
				}

				int clickCount = 0;

				if( button != MouseButtons.None )
				{
					if( wParam == WM_LBUTTONDBLCLK || wParam == WM_RBUTTONDBLCLK )
					{
						clickCount = 2 ;
					}
					else
					{
						clickCount = 1 ;
					}
				}

				//Marshall the data from callback.
				MouseHookStruct MyMouseHookStruct = (MouseHookStruct) Marshal.PtrToStructure( lParam , typeof( MouseHookStruct ) ) ;
				MouseEventArgs e = new MouseEventArgs(
					button ,
					clickCount ,
					MyMouseHookStruct.pt.x ,
					MyMouseHookStruct.pt.y ,
					0 ) ;
				OnMouseActivity( this , e ) ;
			}
			return CallNextHookEx( hMouseHook , nCode , wParam , lParam ) ;
		}
	}
}
